@@ -6,6 +6,7 @@ use strict;
 use Carp qw(croak);
 use IPC::System::Simple qw(capturex systemx runx);
 use Capture::Tiny qw(capture_merged);
+use Passwd::Keyring::OSXKeychain::PasswordTranslate qw(read_security_encoded_passwd);
 
 # TODO: considering we use Capture::Tiny, maybe drop IPC::System::Simple
 #       and move to Capture::Tiny altogether (note that this means 
@@ -18,11 +19,11 @@ Passwd::Keyring::OSXKeychain - Password storage implementation based on OSX/Keyc
 
 =head1 VERSION
 
-Version 0.21
+Version 0.30
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.30';
 
 =head1 WARNING
 
@@ -102,9 +103,9 @@ sub new {
     }
     if($self->{keychain}) {
         # Add .keychain suffix if missing
-        $self->{keychain} .= '.keychain' 
+        $self->{keychain} .= '.keychain'
           unless $self->{keychain} =~ /\.keychain$/;
-    } 
+    }
 
     # Some test operation
     my $reply = capturex($self->_make_sys_args(
@@ -147,6 +148,7 @@ sub set_password {
         ));
 }
 
+# Parser for "-g" find-generic-password variant 
 sub _parse_password_from_find_output {
     my ($text) = @_;
 
@@ -159,9 +161,11 @@ sub _parse_password_from_find_output {
     elsif($text =~ /^ *password: *$/m)  {
         return "";
     }
-
 }
 
+# Set if we use -w (so handle international passwords), unset if -g
+our $USING_ENCODED_OUTPUT = 1;
+
 =head2 get_password($user_name, $realm)
 
 Reads previously stored password for given user in given app.
@@ -172,19 +176,34 @@ If such password can not be found, returns undef.
 sub get_password {
     my ($self, $user_name, $realm) = @_;
 
-    my $reply = capture_merged  {
-        runx(
-            [0, 44],   # Legal exit values. Some CpanTesters report 44 on password not found
+    if($USING_ENCODED_OUTPUT) {
+        my $reply = capturex(
+            [0, 44],
             $self->_make_sys_args(
                 "-q", # quiet
                 "find-generic-password",
                 "-a", $user_name,
                 "-s", $realm,
                 "-D", $self->{group}, # "kind", can be used to match so let be
-                "-g", # display the password
+                "-w", # display (encoded) password only
                ));
-    };
-    return _parse_password_from_find_output($reply);
+        return read_security_encoded_password($reply);
+    }
+    else {
+        my $reply = capture_merged  {
+            runx(
+                [0, 44],   # Legal exit values. Some CpanTesters report 44 on password not found
+                $self->_make_sys_args(
+                    "-q", # quiet
+                    "find-generic-password",
+                    "-a", $user_name,
+                    "-s", $realm,
+                    "-D", $self->{group}, # "kind", can be used to match so let be
+                    "-g", # display the password
+                   ));
+        };
+        return _parse_password_from_find_output($reply);
+    }
 }
 
 =head2 clear_password($user_name, $realm)
